from abc import ABC
from collections import UserDict, UserString, UserList
from datetime import datetime, timedelta
from re import fullmatch
import csv
import copy


class BasePrint(ABC):
    @abstractmethod
    def print_contact_list(self):
        pass

class ContactList(UserDict, BasePrint):

    def __init__(self, name='new_contact_list'):
        self.name = name
        self.data = {}

    def add_contact(self, record):
        self.data[record.name] = record

    #find value in ContactList
    def find(self, find_value):

        finded_names = []
        
        for key, value in self.data.items():
            if find_value in key:
                finded_names.append(key)
                continue
            
            search_values = value['address'] + value['phone'] + value['mail'] + [value['birthday']]

            for i in search_values:
                if find_value in i:
                    finded_names.append(key)
                    break
                
        if not finded_names:
            print(f'No value {find_value} found')
        else:
            print('Finded values in:')
            counter = 0        
            print(' {:_^105}'.format(''))
            print("|{:<10}|{:^27}|{:^27}|{:^27}|{:>10}|".format('name','phone','address','mail','birthday'))
            print(' {:‾^105}'.format(''))
            for i in finded_names:
                print("|{:<10}|{:^27}|{:^27}|{:^27}|{:>10}|".format(i[:10],('/ '.join(self.data[i]['phone']))[:27],('/ '.join(self.data[i]['address']))[:27],('/ '.join(self.data[i]['mail']))[:27],self.data[i]['birthday'].data))
                counter += 1
                if not counter % 10:
                    print(' {:‾^105}'.format(''))
                    input('Press any key to continue')
                    print(' {:_^105}'.format(''))
            print(' {:‾^105}'.format('')) 

    #change name in ContactList
    def change_name(self):
        while True:
            user_input = input('Please enter name to rename/delete: ')
            if user_input not in self.data.keys():
                user_input2 = input('Entered name is not in the contact book.\n1. Try again\n2.return to previous menu?\n')
                if user_input2 == '1':
                    continue
                else:
                    break
            else:
                new_input = input('Enter new name or leave empty to delete contact: ')
                if new_input:
                    self.data[new_input] = self.data.pop(user_input)
                    self.data[new_input].name = new_input
                else:
                    self.data.pop(user_input)
                break

            
       
    #calculate contacts with birthdays from days_from to days_to
    def birthdays(self, days_from, days_to):

        try:
            days_from = int(days_from)
            days_to = int(days_to)
        except:
            return f'Input is not correct. Days should be integer'
    
        if not(isinstance(days_from, int) and isinstance(days_to, int)) or days_from < 0 or days_to < 0 or days_to < days_from:
            return f'Input is not correct. Days should be integer and days from should be less then days to'
    
        date_now = datetime.now()
        delta_from = timedelta(days = days_from)
        delta_to = timedelta(days = days_to)
        
        #dates from - to in datetime
        date_from = date_now + delta_from
        date_to = date_now + delta_to
        
        finded_contacts = []
        for key, value in self.data.items():
            
            #find correct birthday date between from and to
            if date_from.month < value['birthday'].month:
                birthday_date = value['birthday'].date.replace(year = date_now.year)
            elif date_from.month == value['birthday'].month:
                if date_from.day <= value['birthday'].day:
                    birthday_date = value['birthday'].date.replace(year = date_now.year)
                else:
                    birthday_date = value['birthday'].date.replace(year = date_now.year + 1)
            else:
                birthday_date = value['birthday'].date.replace(year = date_now.year + 1)
                
            if date_from < birthday_date < date_to:
                finded_contacts.append(key)
                
        if not finded_contacts:
            print(f'No contacts found')
        else:
            print('Finded birthdays in:')
            counter = 0        
            print(' {:_^105}'.format(''))
            for i in finded_contacts:
                print("|{:<10}|{:^27}|{:^27}|{:^27}|{:>10}|".format(i[:10],('/ '.join(self.data[i]['phone']))[:27],('/ '.join(self.data[i]['address']))[:27],('/ '.join(self.data[i]['mail']))[:27],self.data[i]['birthday'].data))
                counter += 1
                if not counter % 10:
                    print(' {:‾^105}'.format(''))
                    input('Press any key to continue')
                    print(' {:_^105}'.format(''))
            print(' {:‾^105}'.format(''))

    #save contact list
    def save_csv(self):
        user_input = self.name
        with open(f'{user_input}.csv', 'w', newline='') as file:
            field_names = ['name', 'phone', 'address', 'mail', 'birthday']
            writer = csv.DictWriter(file, fieldnames=field_names)
            writer.writeheader()
            for key,value in self.data.items():
                dict_to_write = copy.deepcopy(value)
                dict_to_write['name'] = key
                dict_to_write['phone'] = '&'.join(dict_to_write['phone'])
                dict_to_write['phone'] = dict_to_write['phone'].replace("'",'')
                dict_to_write['address'] = '&'.join(dict_to_write['address'])
                dict_to_write['address'] = dict_to_write['address'].replace("'",'')
                dict_to_write['mail'] = '&'.join(dict_to_write['mail'])
                dict_to_write['mail'] = dict_to_write['mail'].replace("'",'')
                writer.writerow(dict_to_write)

    def load(self):
        user_input = input('Enter the name of your Contact List: ')
        self.name = user_input
        try:
            with open(f'{user_input}.csv', newline='') as file:
                reader = csv.DictReader(file)     
                for row in reader:
                    key = row.pop('name')
                    loaded_record = Record(name = key, phone = row['phone'].split('&'), address = row['address'].split('&'), mail = row['mail'].split('&'), birthday = Birthday(row['birthday']))
                    self.data[key] = loaded_record
            print('Contact list successfully loaded!')
        except FileNotFoundError:
            print('File is not exist.')

    def print_contact_list(self):
        print(f'Contact list: {self.name}')
        print(' {:_^105}'.format(''))
        counter = 0
        
        for i in self.data:
            print("|{:<10}|{:^27}|{:^27}|{:^27}|{:>10}|".format(i[:10],('/ '.join(self.data[i]['phone']))[:27],('/ '.join(self.data[i]['address']))[:27],('/ '.join(self.data[i]['mail']))[:27],self.data[i]['birthday'].data))
            counter += 1
            if not counter % 10:
                print(' {:‾^105}'.format(''))
                input('Press any key to continue')
                print(' {:_^105}'.format(''))
                
        print(' {:‾^105}'.format('')) 

class Birthday(UserString):
    
    @property
    def data(self):
        return self.__data
        
    @data.setter
    def data(self, new_value):
        
        try:
            self.year = int(new_value[6:])
            self.month = int(new_value[3:5])
            self.day = int(new_value[:2])
            self.date = datetime(self.year, self.month, self.day)
            self.__data = new_value
        except ValueError:
            self.__data = ''
            if new_value:
                print(f'Birthday input is not correct. Should be in format: DD.MM.YYYY')
    
class Record(UserDict):
    
    #creating Record dict with all values
    def __init__(self, name = '', phone = '', address = '', mail = '', birthday = 'no_data'):
        self.data = {}
        if not name:
            self.add_name()
        else:
            self.name = name

        if not phone:
            self.add_value('phone')
        else:
            self.data['phone'] = phone

        if not address:
            self.add_value('address')
        else:
            self.data['address'] = address

        if not mail:
            self.add_value('mail')
        else:
            self.data['mail'] = mail

        if birthday == 'no_data':
            self.add_birthday()
        else:
            self.data['birthday'] = birthday
        
    #adding name attribute to the Record
    def add_name(self):
        user_input = input('Enter contact name please: ')
        self.name = user_input
            
    
    #adding phone, mail and address to the Record
    def add_value(self, value):
        while True:
           
            user_input = input(f"Enter the {value} please (If you don't want to enter {value}, leave empty): ")
                    
            if value == 'phone':
                if not self.__phone_check(user_input):
                    continue
            
            if value == 'mail':
                if not self.__mail_check(user_input):
                    continue
            
            if value not in self.data.keys():
                self[value] = [user_input]
            else:
                self[value].append(user_input)

            if not user_input:
                add_more = 'no'
            else:
                add_more = input(f'Do you want to enter another {value}? (yes|no): ')
                
            if add_more == 'yes':
                continue
            else:
                break
    
    #phone check
    def __phone_check(self, phone):

        PHONE_CHECK = '[+]?380(93|67|63|50|95|66|97|68|73|96|98|99)\d{7}'

        if fullmatch(PHONE_CHECK, phone) or not phone:
            return True
        else:
            print('Phone number is not correct. Should start from +380 or 380, have correct operator number and 12 digits')
            return False
    
    #mail check
    def __mail_check(self, mail):

        MAIL_CHECK = '[a-zA-Z0-9_]{2,15}[@][a-z]{1,10}\.[a-z]{2,4}'

        if fullmatch(MAIL_CHECK, mail) or not mail:
            return True
        else:
            print('Mail is not correct.')
            return False
    
    #birthday adding
    def add_birthday(self):
        while True:
           
            user_input = input('Enter the birthday please in format "DD.MM.YYYY"(If you do not want to enter birthday, leave empty): ')
            self.data['birthday'] = Birthday(user_input)
            
            if self.data['birthday'] or not user_input:
                break
    
    #change any value in Record except name
    def change_value(self):
        
        CORRECT_INPUT = ('1', 'phone', '2', 'address', '3', 'mail', '4', 'birthday','5','back')

        while True:
            user_input = input("What value do you want to change/delete?\n1. Phone\n2. Address\n3. Mail\n4. Birthday\n5. Back\n")

            if user_input  not in CORRECT_INPUT:
                print('Please choose correct option from the list: ')
                continue
            elif user_input in ('5', 'back'):
                break
            elif user_input in ('4', 'birthday'):
                self.add_birthday()
                break
            else:

                if user_input in ('1', 'phone'):
                    value = 'phone'
                elif user_input in ('2','address'):
                    value = 'address'
                else:
                    value = 'mail'

                print(self.data[value])

                user_input = input(f'Choose the number of {value} to change/delete. If you want to delete all values, type "clear": ')

                if user_input == 'clear':
                    self.data[value].clear()
                    print(f'{value} Successfully cleared!')
                    break

                data_input = input(f'Enter the correct {value} or leave empty if you want to delete {value}: ')

                if value == 'phone':
                    if not self.__phone_check(data_input):
                        continue
            
                if value == 'mail':
                    if not self.__mail_check(data_input):
                        continue
                if data_input:
                    self.data[value][int(user_input)-1] = data_input
                else:
                    self.data[value].pop(int(user_input)-1)
                break



import ProjectHelper.contacts_list
import ProjectHelper.sort_files
import ProjectHelper.mynote
from datetime import datetime

def log(action):

    current_time = datetime.strftime(datetime.now(), '%H:%M:%S')
    message = f'[{current_time}] {action}'
    print('_'*100)
    print(message)
    print('‾'*100)
    with open('logs.txt', 'a') as file:
        file.write(f'{message}\n')

def readme():
    print('''This is the helper with the functions:
1. Create and work with Contacts lists.
2. Create and work with the notes.
3. Sorting files in the folders.

You can choose your options in the console.
Thank you for using our Helper.''')
        
def main():
    
    log("Hello, I'm your personal helper. What can I do for you today?")
    while True:
        print('_'*100)
        user_input = input(f"1. Work with contacts list\n2. Work with notes\n3. Sort your folders\n4. Help\n5. Exit\n{'‾'*100}\n")

        if user_input in ('exit', '5'):
            log('Bye!')
            break
        elif user_input == '4':
            readme()
        elif user_input == '3':
            ProjectHelper.sort_files.main()
        elif user_input == '2':
            n = ProjectHelper.mynote.NoteList()
            while True:
                print('_'*100)
                user_input2 = input(f'What do you want to do with notes?\n1. Create new note\n2. Change note info\n\
3. View notes\n4. Delete note\n5. Find note by tag\n6. Find value in notes \n7. Save changes\n\
8. Back\n{"‾"*100}\n')
                if user_input2 == '1':
                    name = input('Enter the note name: ')
                    data = input('Enter your note: ')
                    tags = input('Enter your tags separated by " ": ')
                    new_note = ProjectHelper.mynote.Note(name, data, tags)
                    n.add_note(new_note)
                    log('Note has been created!')
                    
                elif user_input2 == '2':
                    n.print_notelist()
                    name_note_to_change = input('Choose the name of the note to change: ')
                    index_note = 0
                    index_flag = False
                    for note in n.data:
                        if note.name == name_note_to_change:
                            index_flag = True
                            break
                        index_note += 1
                    if not index_flag:
                        print('Note with this name does not exist')
                        continue
                        
                    while True:
                        print('_'*100)
                        user_input3 = input(f'What exactly do you want to change?\n1. Change name\n2. Change note\n\
3. Change tag\n4. Add tag\n5. Back\n{"‾"*100}\n')
                        if user_input3 == '1':
                            name_change = input('Enter new name: ')
                            n.data[index_note].change_name(name_change)
                        elif user_input3 == '2':
                            note_change = input('Enter new note: ')
                            n.data[index_note].change_data(note_change)
                        elif user_input3 == '3':
                            n.data[index_note].change_tag()
                        elif user_input3 == '4':
                            new_tag = input('Enter new tag: ')
                            n.data[index_note].add_tag(new_tag)
                        elif user_input3 == '5':
                            break
                        else:
                            print('Please choose correct option')
                            
                elif user_input2 == '3':
                    n.print_notelist()
                elif user_input2 == '4':
                    n.print_notelist()
                    name_del = input('Enter the name of the note to delete: ')
                    n.delete_note(name_del)
                elif user_input2 == '5':
                    tag_to_find = input('Enter the tag to find in the notes: ')
                    n.find_tags(tag_to_find)
                elif user_input2 == '6':
                    name_to_find = input('Enter the value to find in the notes: ')
                    n.find(name_to_find)    
                elif user_input2 == '7':
                    n.save_notelist()
                    log('Notes has been saved!')
                elif user_input2 == '8':
                    break
                else:
                    print('Please choose correct option')

        elif user_input == '1':
            active_list_flag = False
            while True:
                print('_'*100)
                user_input2 = input(f'What do you want to do with contacts list?\n1. Load contacts list\n2. Save contacts list\n3. New contacts list\n\
4. View contact list\n5. Create contact\n6. Change/delete name of contact\n7. Add/change/delete contact information\n8. Search value in contacts\n\
9. Find contacts with birthdays between two dates\n10. Back\n{"‾"*100}\n')
                
                if user_input2 == '1':
                    current_contact_list = ProjectHelper.contacts_list.ContactList()
                    current_contact_list.load()
                    active_list_flag = True

                elif user_input2 == '2':
                    if active_list_flag:
                        current_contact_list.save_csv()
                        log('Contact list successfully saved!')
                    else:
                        log('You are not working with contacts list now. Please load or create new one.')

                elif user_input2 == '3':
                    if active_list_flag:
                        user_input3 = input('Do you want to save current contact list? (yes|no): ')
                        if user_input3 == 'yes':
                            current_contact_list.save_csv()
                            log('Contact list successfully saved!')

                    list_name = input('Enter contact list name: ')
                    current_contact_list = ProjectHelper.contacts_list.ContactList(list_name)
                    active_list_flag = True
                    log('New contact list created!')

                elif user_input2 == '4':
                    if active_list_flag:
                        current_contact_list.print_contact_list()
                    else:
                        log('You are not working with contacts list now. Please load or create new one.')

                elif user_input2 == '5':
                    if active_list_flag:
                        new_contact = ProjectHelper.contacts_list.Record()
                        if new_contact.name in current_contact_list.keys():
                            print('User name is occupied. Please try another name.')
                            continue
                        current_contact_list.add_contact(new_contact)
                        log('Contact successfully created!')
                    else:
                        log('You are not working with contacts list now. Please load or create new one.')

                elif user_input2 == '6':
                    if active_list_flag:
                        current_contact_list.change_name()
                    else:
                        log('You are not working with contacts list now. Please load or create new one.')

                elif user_input2 == '7':
                    if active_list_flag:
                        print(list(current_contact_list.keys()))
                        name_to_change = input('Choose the contact to change/delete: ')
                        if name_to_change not in current_contact_list.keys():
                            log('Name is not in contact list')
                            continue
                        print('_'*100)    
                        add_or_change = input(f'What exactly do you want to do?\n1. Add new info\n2. Change/delete info\n3. Back\n{"‾"*100}\n')
                        if add_or_change == '1':
                            print('_'*100)
                            what_to_add = input(f'What exactly do you want to add?\n1. Phone\n2. Address\n3. Mail\n4. Birthday\n5. Back\n{"‾"*100}\n')
                            if what_to_add == '1':
                                current_contact_list[name_to_change].add_value('phone')
                            elif what_to_add == '2':
                                current_contact_list[name_to_change].add_value('address')
                            elif what_to_add == '3':
                                current_contact_list[name_to_change].add_value('mail')
                            elif what_to_add == '4':
                                if not current_contact_list[name_to_change]['birthday']:
                                    current_contact_list[name_to_change].add_birthday()
                                else:
                                    log('Birthday has been already added')
                            elif what_to_add =='5':
                                pass
                            else:
                                log('Please choose correct option.')
                        
                        elif add_or_change == '2':
                            current_contact_list[name_to_change].change_value()
                    else:
                        log('You are not working with contacts list now. Please load or create new one.')


                elif user_input2 == '8':
                    if active_list_flag:
                        find_value = input('Enter the value to find: ')
                        current_contact_list.find(find_value)
                    else:
                        log('You are not working with contacts list now. Please load or create new one.')

                elif user_input2 == '9':
                    if active_list_flag:
                        days_from = input('Enter the q-ty days from today to search from: ')
                        days_to = input('Enter the q-ty days from today to search to: ')
                        current_contact_list.birthdays(days_from, days_to)
                    else:
                        log('You are not working with contacts list now. Please load or create new one.')

                elif user_input2 == '10':
                    break

                else:
                    log('Please choose correct option.')
        else:
            log('Please choose correct option.')

if __name__ == '__main__':
    main()



import os
import shutil
import sys

GLOBAL_PATH = ''

# Folders to create
folders = ['archive', 'audio', 'documents', 'images', 'video', 'other']

# Extentions of known files
archive_ext = ['.7z', '.ace', '.arj', 'bz2', '.cab', '.cbr', '.d8t', '.deb', '.gz', '.gzip', '.gztar', '.jar',
               '.pak', '.php', '.pkg', '.rar', '.rpm', '.sh', '.sit', '.sitx', '.tar', '.tar-gz', '.tgz', '.xz', '.zip', '.zipx']
audio_ext = ['.aac', '.ac3', '.aif', '.aiff', '.amr', '.ape', '.asf', '.asx', '.au', '.aud', '.flac', '.iff', '.m3u', '.m3u8',
             '.m4a', '.m4b', '.m4p', '.m4r', '.mid', '.midi', '.mod', '.mp3', '.mpa', '.ogg', '.ra', '.ram', '.sib', '.wav', '.wm', '.wma']
docs_ext = ['.djvu', '.doc', '.docm', '.docx', '.dot', '.dotm', '.dotx', '.epub', '.fb2', '.ibooks', '.indd', '.key', '.mobi', '.mso', '.ods', '.odt',
            '.one', '.oxps', '.pages', '.pdf', '.pkg', '.pot', '.potm', '.potx', '.pps', '.ppsm', '.ppsx', '.ppt', '.pptm', '.pptx', '.ps', '.pub',
            '.rtf', '.sldm', '.vcard', '.vcf', '.txt', '.wpd', '.wps', '.xar', '.xlr', '.xls', '.xlsb', '.xlsm', '.xlsx', '.xlt', '.xltm', '.xltx', '.xps']
imgs_ext = ['.ai', '.bmp', '.cdd', '.cdr', '.dds', '.dng', '.eps', '.gbr', '.gif', '.iso', '.jpeg', '.jpg', '.kdc', '.mng', '.msp', '.pcx', '.pdd',
            'pdp', '.png', '.pot', '.ps', '.psb', '.psd', '.pspimage', '.raw', '.svg', '.tga', '.thm', '.tif', '.tiff', '.vsd', '.vst', '.wmf', '.xcf']
video_ext = ['.3g2', '.3gp', '.avi', '.f4v', '.flv', '.m4v', '.mkv', '.mod', '.mov', '.mp4', '.mpeg',
             '.mpg', '.mts', '.rm', '.rmvb', '.srt', '.swf', '.ts', '.vob', '.webm', '.wmv', '.yuv']

# Normalization of file names


def normalize(string):

    tr_map = {ord('а'): 'a', ord('б'): 'b', ord('в'): 'v', ord('г'): 'g', ord('д'): 'd', ord('е'): 'e', ord('є'): 'ye', ord('ж'): 'zh', ord('з'): 'z', ord('и'): 'y',
              ord('і'): 'i', ord('ї'): 'yi', ord('й'): 'y', ord('к'): 'k', ord('л'): 'l', ord('м'): 'm', ord('н'): 'n', ord('о'): 'o', ord('п'): 'p', ord('р'): 'r',
              ord('с'): 's', ord('т'): 't', ord('у'): 'u', ord('ф'): 'f', ord('х'): 'kh', ord('ц'): 'ts', ord('ч'): 'ch', ord('ш'): 'sh', ord('щ'): 'shch', ord('ы'): 'y',
              ord('ъ'): '', ord('ь'): '', ord('э'): 'e', ord('ю'): 'yu', ord('я'): 'ya', ord('ё'): 'yo', ord('А'): 'A', ord('Б'): 'B', ord('В'): 'V', ord('Г'): 'G',
              ord('Д'): 'D', ord('Е'): 'E', ord('Є'): 'Ye', ord('Ж'): 'Zh', ord('З'): 'Z', ord('И'): 'Y', ord('І'): 'I', ord('Ї'): 'Yi', ord('Й'): 'Y', ord('К'): 'K',
              ord('Л'): 'L', ord('М'): 'M', ord('Н'): 'N', ord('О'): 'O', ord('П'): 'P', ord('Р'): 'R', ord('С'): 'S', ord('Т'): 'T', ord('У'): 'U', ord('Ф'): 'F',
              ord('Х'): 'Kh', ord('Ц'): 'Ts', ord('Ч'): 'Ch', ord('Ш'): 'Sh', ord('Щ'): 'Shch', ord('Ы'): 'Y', ord('Ъ'): '', ord('Ь'): '', ord('Э'): 'E', ord('Ю'): 'Yu',
              ord('Я'): 'Ya', ord('Ё'): 'Yo'}

    normalized = []

    for c in string:
        if not c.isalpha() and not c.isdigit():
            c = '_'
            normalized.append(c)
        else:
            c = c.translate(tr_map)
            normalized.append(c)

    return ''.join(normalized)


# Folders creation
def create_folder(folders):
    for name in folders:
        directory = os.path.join(GLOBAL_PATH, name)
        try:
            os.stat(directory)
        except:
            os.mkdir(directory)


# Files move
def move_files(files_info):
    create_folder(folders)
    info = files_info.split(";")
    src = os.path.join(info[1], info[2]+info[3])
    destination = os.path.join(
        GLOBAL_PATH, info[0], normalize(info[2])+info[3])
    shutil.move(src, destination)
    try:
        os.rmdir(info[1])
    except OSError:
        pass


# Sort files in dirs
def sort_files(collection, path, nest_deep):
    for file in collection:
        name, ext = os.path.splitext(file)
        if ext.lower() in archive_ext:
            move_files(f'archive;{path};{name};{ext}')
        elif ext.lower() in audio_ext:
            move_files(f'audio;{path};{name};{ext}')
        elif ext.lower() in docs_ext:
            move_files(f'documents;{path};{name};{ext}')
        elif ext.lower() in imgs_ext:
            move_files(f'images;{path};{name};{ext}')
        elif ext.lower() in video_ext:
            move_files(f'video;{path};{name};{ext}')
        else:
            if os.path.isfile(os.path.join(path, file)):
                move_files(f'other;{path};{name};{ext}')


# Structures to find files
def grab_path(path, nest_deep=0):
    collection = []
    for file in os.listdir(path):
        if os.path.isdir(os.path.join(path, file)):
            grab_path(os.path.join(path, file), nest_deep + 1)
        else:
            collection.append(file)

    sort_files(collection, path, nest_deep)


def main():
    global GLOBAL_PATH
    while True:
        user_input = input('Do you want to sort current folder? (yes/no) ')
        if user_input.lower().strip() == 'yes':
            GLOBAL_PATH = os.getcwd()
            break
        elif user_input.lower().strip() == 'no':
            GLOBAL_PATH = input('Please enter the folder to do sorting: ')
            break
        else:
            print('Please choose correct option.')
    try:
        grab_path(GLOBAL_PATH)
        
        print('_{:_^155}'.format(''))

        print(f"Sorted directory:\t {GLOBAL_PATH}")
        print('_{:_^155}'.format(''))
        for i in folders:
            print("{:<10}|{:<144}".format(i.upper(), (' / '.join(
                os.listdir(os.path.join(GLOBAL_PATH, i))))))
        print('_{:_^155}'.format(''))
    except:
        print('Something went wrong... Your input is incorrect')


if __name__ == '__main__':
    main()



from abc import ABC
from datetime import datetime
import csv


class BasePrint(ABC):
    @abstractmethod
    def print_notelist(self):
        pass

class Note:
    
    def __init__(self, name, data, tags, current_time = datetime.strftime(datetime.now(), '%d.%m.%Y %H:%M:%S')):
        self.name = name
        self.data = data
        self.tags = tags.split()
        self.current_time = current_time
        
    def add_tag(self, tag):
        self.tags.append(tag)

    def change_name(self, name):
        if not name:
            print('Note name cannot be empty')
        else:
            self.name = name
            self.current_time = datetime.strftime(datetime.now(), '%d.%m.%Y %H:%M:%S')
            
    def change_data(self, data):
        self.data = data
        self.current_time = datetime.strftime(datetime.now(), '%d.%m.%Y %H:%M:%S')        
    
    def change_tag(self):
        print(self.tags)
        user_input = input('Choose tag to change: ')
        if user_input in self.tags:
            user_input2 = input('Enter new tag: ')
            index_to_change = self.tags.index(user_input)
            self.tags[index_to_change] = user_input2
            self.current_time = datetime.strftime(datetime.now(), '%d.%m.%Y %H:%M:%S')        
            print('Tag successfully changed!')
            
    def print_note(self):
        print("|{:<20}|{:^15}|{:^25}|{}|".format(self.current_time,self.name, ' '.join(self.tags), self.data))
        print(' {:‾^105}'.format('')) 

class NoteList(BasePrint):
    
    def __init__(self):
        self.data = []
        self.load_notelist()
        
    def add_note(self, note):
        self.data.append(note)
        
    def delete_note(self, note_name):
        for note in self.data:
            if note.name == note_name:
                self.data.remove(note)
                break
        
    def find(self, search_value):
        find_flag = False    
        for i in self.data:
            if search_value in i.name or search_value in i.data:
                i.print_note()
                find_flag = True
        if not find_flag:
            print('Data was not founded in the notes ')
    
    def find_tags(self, search_value):
        find_flag = False
        for i in self.data:
            if search_value in i.tags:
                i.print_note()
                find_flag = True
                
        if not find_flag:
            print('Tag was not founded in the notes')
                
    def save_notelist(self):
        with open('notes.csv', 'w', newline='') as file:
            field_names = ['time', 'name', 'tags', 'note']
            writer = csv.DictWriter(file, fieldnames=field_names)
            writer.writeheader()
            dict_to_write = {}
            for note in self.data:
                dict_to_write['time'] = note.current_time
                dict_to_write['name'] = note.name
                dict_to_write['tags'] = ' '.join(note.tags)
                dict_to_write['note'] = note.data
                writer.writerow(dict_to_write)
                
    def load_notelist(self):
        try:
            with open('notes.csv', 'r', newline = '') as file:
                reader = csv.DictReader(file)
                for row in reader:
                    new_note = Note(row['name'], row['note'], row['tags'], row['time'])
                    self.data.append(new_note)
        except FileNotFoundError:
            self.data = []
                
    def print_notelist(self):
        print(' {:_^105}'.format(''))
        for note in self.data:
            note.print_note()



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(name='personal_assistant',
                 version='0.0.1',
                 author='Yaroslav Maniukh, Valeriy Topchiy, Polina Yarova',
                 author_email='manyukhy@gmail.com, wellkswell@gmail.com, polinaya777@gmail.com',
                 description='Console script for working with Contacts lists, Notes and sorting files in the folders',
                 long_description=long_description,
                 long_description_content_type="text/markdown",
                 url='https://github.com/Personal-Assistant-Project/ProjectHelper',
                 keywords="personal assistant helper",
                 license='MIT',
                 classifiers=[
                     "Programming Language :: Python :: 3",
                     "License :: OSI Approved :: MIT License",
                     "Operating System :: OS Independent",
                 ],
                 entry_points={'console_scripts': [
                     'helper-folder = ProjectHelper.main:main']},
                 packages=setuptools.find_packages(),
                 include_package_data=True,
                 python_requires=">=3.6",
                 )
